function ord = aivga_show(InStruct, data)
%function [ordering]=aivga_show(InStruct[, data])
%  This function is used to visualise the results of AIVGA given in InStruct.
%  The optional second argument is the original data set given to AIVGA,
%  it is used to extract names of the variables if they were given to
%  aivga_import_data.
%  The output is the order of variables used in visualisation.


% This file is a part of the AIVGA program
%
% Copyright (C) 2006-2007 Esa Alhoniemi, Antti Honkela and Jeremias Seppa
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2, or (at your option)
% any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License (included in file License.txt in the
% program package) for more details.

draw=1;

ch(1:2,:)=InStruct.moves;
ch(3,:)=InStruct.cost;
gh=InStruct.groupings;

[dummy,dim]=size(ch);
ord=1:dim;

for n=(dim-1):-1:1
  g=gh{n};
  if (n==1); g=gh{n}{n};end  %strange :( ..
  a=ch(1,n+1);b=ch(2,n+1);
  ordblockvars=[g{a} g{b}];
  blockstart=find(ord==min(ordblockvars));
  ord(blockstart:(blockstart+length(ordblockvars)-1))=ordblockvars;
end
if draw
  clf
  subplot(1,3,1:2)
  hold on
  for v=1:dim
    plot([0 1],[v v])
  end
  for n=2:(dim)
    g=gh{n};  
    if n<dim  % draw not the last horizontal stick
      for gi=1:(dim-n+1)
	a=find(g{gi}(1)==ord);
	b=a+length(g{gi})-1;
	plot([n-1 n],[1 1].*(mean([a b])));
      end
    end
    g=gh{n-1};
    if (n==2); g=gh{n-1}{1};end  %strange :( ..
    meana=find(g{ch(1,n)}(1)==ord)+0.5*(length(g{ch(1,n)})-1);
    meanb=find(g{ch(2,n)}(1)==ord)+0.5*(length(g{ch(2,n)})-1);
    plot([n-1 n-1],[meana,meanb]);
  end
  set(gca,'YTick',1:dim)
  if nargin > 1,
    set(gca,'YTickLabel', data.var_names(ord))
  else
    set(gca,'YTickLabel',ord)
  end
  axis([0 dim .5 dim+.5]);
  [foo, I] = min(ch(3, :));
  plot([I-.5, I-.5], [.5, dim+.5], '--');
  set(gca, 'XTick', []);
  ylabel('Variable')
  subplot(1,3,3)
  plot(1:dim,fliplr(ch(3,:)))
  v = axis;
  axis([1 dim v(3:4)]);
  hold on
  plot(dim-I+1*[1 1], v(3:4), '--');
  ylabel('Cost');
  xlabel('# of groups');
end

